import pytest

# pytest.importorskip("kimmdy-grappa")
pytest.importorskip("grappa")

import os
import math
from pathlib import Path
from copy import deepcopy

from kimmdy.topology.topology import Topology
from kimmdy.topology.atomic import MultipleDihedrals
from kimmdy.parsing import read_top


from grappa_interface import GrappaInterface


def test_parameterize_topology(tmp_path):
    os.chdir(tmp_path.resolve())
    parameterizer = GrappaInterface()
    top = Topology(read_top(Path(__file__).parent / "Ala_out.top"), parameterizer)

    curr_top = deepcopy(top)
    curr_top.needs_parameterization = True
    curr_top.update_parameters()

    assert top != curr_top

    # if charge assignment gets into grappa
    # assert len(curr_top.atoms) == 21
    # charge_sum = 0
    # for atom in curr_top.atoms.values():
    #     charge_sum += int(atom.charge)
    # assert math.isclose(charge_sum,0,rel_tol=1e-5)

    assert len(curr_top.bonds) == 20
    for bond in curr_top.bonds.values():
        assert bond.c0 is not None
        assert bond.c1 is not None

    assert len(curr_top.angles) == 33
    for angle in curr_top.angles.values():
        assert angle.c0 is not None
        assert angle.c1 is not None

    assert len(curr_top.proper_dihedrals) == 34
    for multiple_dihedral in curr_top.proper_dihedrals.values():
        assert len(multiple_dihedral.dihedrals) in [3, 6]

    assert len(curr_top.improper_dihedrals) == 15
    for multiple_dihedral in curr_top.improper_dihedrals.values():
        assert isinstance(multiple_dihedral, MultipleDihedrals)
        assert len(multiple_dihedral.dihedrals) == 3



import logging
from typing import Optional

from kimmdy.plugins import Parameterizer
from kimmdy.topology.topology import Topology

from grappa.grappa import Grappa
from grappa.utils.kimmdy_utils import KimmdyGrappaParameterizer

logger = logging.getLogger("kimmdy.grappa_interface")


class GrappaInterface(Parameterizer):
    """
    Wrapper of the KimmdyGrappaParameterizer used in grappa. Initialised with a tag instead of a model.
    """

    def __init__(
        self, *args, grappa_tag: str = "latest", charge_model: str = "amber99", **kwargs
    ):
        super().__init__(*args, **kwargs)
        logger.info(f"Instantiating Grappa with tag '{grappa_tag}'.")
        grappa_instance = Grappa.from_tag(grappa_tag)
        self.kimmdy_grappa_parameterizer = KimmdyGrappaParameterizer(
            grappa_instance=grappa_instance,
            charge_model=charge_model,
        )

    def parameterize_topology(
        self, current_topology: Topology, focus_nrs: Optional[set[str]] = None
    ) -> Topology:
        return self.kimmdy_grappa_parameterizer.parameterize_topology(
            current_topology=current_topology, focus_nrs=focus_nrs
        )
